*==============================================================================
* Program:				SELECTIONVSRANGE.PRG
* Purpose:				Test relative speed of an automation task using 
*                       selection and range objects.
* From:				    Office Automation Tips, Tricks and Traps
* Copyright:			(c) 2010 Tamar E. Granor, Ph.D.
* Last revision:		09/07/10
*==============================================================================
 
LPARAMETERS lVisible 
	* Pass .T. to show Word as your work and to see the difference having
	* Word visible makes in these tests.

LOCAL oWord, oDoc, cFile

oWord = CREATEOBJECT("Word.Application")
IF m.lVisible
	oWord.Visible = .T.
ENDIF 

* This code assumes the selected document has a table with at least three columns.
* It will fail if any other kind of file is selected.
cFile = GETFILE()

oDoc = oWord.Documents.Add(m.cFile)

LOCAL nStart, nEnd, nRow, nRowCount
LOCAL lInsideParens, nWordCount, nWord, cWord

* The task performed in each test is to go through 
* column 3 of the table, select each non-parenthesized 
* word and bold it.

* Test 1a: Use the selection object. Each time through the 
*          loop to process a word, reselect the entire cell, 
*          then pick out the word you want.

nStart = SECONDS()
oDoc.Tables[1].Select()

nRowCount = oWord.Selection.Rows.Count

FOR nRow = 2 TO m.nRowCount
	oDoc.Tables[1].Cell(m.nRow, 3).Select()
	nWordCount = oWord.Selection.Words.Count
	
	FOR nWord = 1 TO m.nWordCount-1
		* Stop before the last word, which is just punctuation
		* Select just one word
		oWord.Selection.Words[m.nWord].Select()
		cWord = oWord.Selection.Text
		DO CASE 
		CASE m.cWord = "("
			lInsideParens = .T.
			
		CASE m.cWord = ")"
			lInsideParens = .F.
			
		CASE INLIST(m.cWord, ",", CHR(13))
			* Do nothing
			
		CASE NOT m.lInsideParens
			* Bold it
			oWord.Selection.Font.Bold = .T.
	
		OTHERWISE
			* Nothing to do
		ENDCASE
		
		* Reselect the whole cell
		oDoc.Tables[1].Cell(m.nRow, 3).Select()
	ENDFOR 
ENDFOR 

nEnd = SECONDS()
?"1a: Using brute force selection, bolding non-paren words in column 3 takes", m.nEnd - m.nStart

* Close the document without saving changes
oDoc.Close(0)

* Test 1b: Use the selection object, but use collapse 
*          and expand to modify selection
oDoc = oWord.Documents.Add(m.cFile)

nStart = SECONDS()
oDoc.Tables[1].Select()

nRowCount = oWord.Selection.Rows.Count

LOCAL lInsideParens
FOR nRow = 2 TO m.nRowCount
	oDoc.Tables[1].Cell(m.nRow, 3).Select()
	nWordCount = oWord.Selection.Words.Count
	
	oWord.Selection.Collapse(1) && Collapse to beginning
	
	FOR nWord = 1 TO m.nWordCount-1
		* Stop before the last word, which is just punctuation
		* Select just one word
		oWord.Selection.Expand(2) 
		cWord = oWord.Selection.Text
		DO CASE 
		CASE m.cWord = "("
			lInsideParens = .T.
			
		CASE m.cWord = ")"
			lInsideParens = .F.
			
		CASE INLIST(m.cWord, ",", CHR(13))
			* Do nothing
			
		CASE NOT m.lInsideParens
			* Bold it
			oWord.Selection.Font.Bold = .T.
	
		OTHERWISE
			* Nothing to do
		ENDCASE
		
		* Collapse the selection
		oWord.Selection.Collapse(0)
	ENDFOR 
ENDFOR 

nEnd = SECONDS()
?"1b: Using selection and collapse/expand, bolding non-paren words in column 3 takes", m.nEnd - m.nStart

* Close the document without saving changes
oDoc.Close(0)


* Test 2a: Use ranges and other object references
oDoc = oWord.Documents.Add(m.cFile)

nStart = SECONDS()
FOR nRow = 2 TO oDoc.Tables[1].Rows.Count
	oRange = oDoc.Tables[1].Cell(m.nRow, 3).Range()
	
	FOR nWord = 1 TO oRange.Words.Count-1
		oWordRange = oRange.Words[m.nWord]
		cWord = oWordRange.Text
		DO CASE 
		CASE m.cWord = "("
			lInsideParens = .T.
			
		CASE m.cWord = ")"
			lInsideParens = .F.
			
		CASE INLIST(m.cWord, ",", CHR(13))
			* Do nothing
			
		CASE NOT m.lInsideParens
			* Bold it
			oWordRange.Font.Bold = .T.
	
		OTHERWISE
			* Nothing to do
		ENDCASE
	ENDFOR
ENDFOR
nEnd = SECONDS()

?"2a: Using ranges, bolding non-paren words in column 3 takes ", m.nEnd-m.nStart

* Close the document without saving changes
oDoc.close(0)

* Test 2b: Use ranges and collapse/expand
oDoc = oWord.Documents.Add(m.cFile)

nStart = SECONDS()
FOR nRow = 2 TO oDoc.Tables[1].Rows.Count
	oRange = oDoc.Tables[1].Cell(m.nRow, 3).Range()
	nWordCount = oRange.Words.Count
	
	oRange.Collapse(1)
	FOR nWord = 1 TO m.nWordCount-1
		oRange.Expand(2)
		cWord = oRange.Text
		DO CASE 
		CASE m.cWord = "("
			lInsideParens = .T.
			
		CASE m.cWord = ")"
			lInsideParens = .F.
			
		CASE INLIST(m.cWord, ",", CHR(13))
			* Do nothing
			
		CASE NOT m.lInsideParens
			* Bold it
			oRange.Font.Bold = .T.
	
		OTHERWISE
			* Nothing to do
		ENDCASE
		
		oRange.Collapse(0)
	ENDFOR
ENDFOR
nEnd = SECONDS()

?"2b: Using ranges and collapse/Expand, bolding non-paren words in column 3 takes ", m.nEnd-m.nStart

* Close the document without saving changes
oDoc.close(0)

* Test 3: Copy entire cell to VFP, parse there, and then 
*         process as needed.

oDoc = oWord.Documents.Add(m.cFile)

nStart = SECONDS()
FOR nRow = 2 TO oDoc.Tables[1].Rows.Count
	oRange = oDoc.Tables[1].Cell(m.nRow, 3).Range()
	cCellContent = oRange.Text
	
	* Break into words
	nWordCount = ALINES(aWords, m.cCellContent, ",")
		
	* Word counts words differently, so need to keep track.
	LOCAL nWordWordNum, lHasParen
	nWordWordNum = 0
	lHasParen = .F.
		
	FOR nWord = 1 TO m.nWordCount
		cWord = aWords[m.nWord]
		nWordWordNum = m.nWordWordNum + 1 
		
		* Check whether this one has parenthetical
		IF "(" $ m.cWord
			lHasParen = .t.
		ELSE
			lHasParen = .F.	
		ENDIF 
	
		* Highlight this one
		oRange.Words[m.nWordWordNum].Font.Bold = .T.
	
		* Update count of words in Word.	
		IF m.lHasParen
			* Left paren, year, right paren with comma
			nWordWordNum = m.nWordWordNum + 3
		ELSE 
			* Just the comma
			nWordWordNum = m.nWordWordNum + 1 
		ENDIF
		
	ENDFOR 
		
ENDFOR
nEnd = SECONDS()

?"3: Using VFP to parse, bolding non-paren words in column 3 takes ", m.nEnd-m.nStart

* Close the document without saving changes
oDoc.close(0)
* Close Word.
oWord.Quit()

RETURN

